﻿using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;

namespace WOLPowerManager
{
  public class Utilities
  {


    #region Array-Operations

    /// <summary>
    /// Method for resizing a multi dimensional array
    /// </summary>
    /// <param name="original">Original array you want to resize</param>
    /// <param name="cols"># of columns in the new array</param>
    /// <param name="rows"># of rows in the new array</param>
    public static string[,] ResizeArray(string[,] original, int cols, int rows)    
    {    
        //create a new 2 dimensional array with    
        //the size we want    
        string[,] newArray = new string[rows, cols];   
        //copy the contents of the old array to the new one    
        Array.Copy(original, newArray, original.Length);
        //set the original to the new array
        return  newArray;  
    }

    /// <summary>
    /// Method for remove a specific entry of an 2D-array
    /// </summary>
    /// <param name="original">Original array you want to remove a line</param>
    /// <param name="at">"index of the line to delete</param>
    public static string[,] RemoveAt(string [,] original, int at)
    {
        for (int i = at; i < original.GetLength(0)-1; i++)
        {
            for (int j = 0; j < original.GetLength(1); j++)
			{
			 original[i,j]=original[i+1,j];
			}
        }
        //create a new 2 dimensional array with    
        //the size we want    
        string[,] newArray = new string[original.GetLength(0) - 1, original.GetLength(1)];
        //copy the contents of the old array to the new one    
        Array.Copy(original, newArray, original.Length - original.GetLength(1));
        //set the original to the new array
        return newArray;  
    }
    #endregion


#region Network

    //Wandelt den angegebenen Ip-String in einen Int32-Wert um   
    [DllImport("Ws2_32.dll")]
    private static extern Int32 inet_addr(string ip);

    //Holt die Mac-Adresse der angegebenen Ip-Adresse   
    //Die Ip-Adresse muss im Int32-Format vorliegen.   
    [DllImport("Iphlpapi.dll")]
    private static extern int SendARP(Int32 dest, Int32 host, ref Int64 mac, ref Int32 len);

    //Diese Methode holt die Mac-Adresse einer angegebenen Ip-Adresse   
    //LocalIp ist die Ip-Adresse des eigenen PCs und kann ein Leerstring sein   
    //remoteIp ist die Ip-Adresse des anderen PCs und muss die durch Punkte getrennte
    //Ip-Adresse beinhalten   
    public static Int64 getRemoteMAC(string remoteIP)
    {
        //Umwandlung der Ip-Adressen in Int32-Werte         
        Int32 ldest = inet_addr(remoteIP);

        try
        {
            Int64 macinfo = 0; //Dient zur Speicherung der Mac-Adresse             
            Int32 len = 6; //Länge             

            int res = SendARP(ldest, 0, ref macinfo, ref len); //Mac-Adresse holen

            return macinfo; //Int64 mit Mac-Adresse zurückgeben         
        }
        catch (Exception e)
        {
            string PLUGIN_NAME = "WOLPowerManager";
            MediaPortal.GUI.Library.Log.Error("{0}:getRemoteMAC(): {1}", PLUGIN_NAME, e.ToString());
            return 0;
        }
    }

    //Mit Hilfe dieser Methode kann die Mac-Adresse im Int64-Format leicht in
    //das übliche durch - getrennte Format umgewandelt werden     
    public static string formatMac(Int64 mac, char separator)
    {
        //Wenn Mac-Adresse 0 --> Umwandlung in 00-00-00-00-00-00 und im Anschluss exit
        if (mac <= 0)
            return "00:00:00:00:00:00";

        //Die Int64-Mac-Adresse wird in einen Hex-String umgewandelt, mit 0 aufgefüllt
        //und in ein Byte-Array aufgesplittet         
        char[] oldmac = Convert.ToString(mac, 16).PadLeft(12, '0').ToCharArray();

        System.Text.StringBuilder newMac = new System.Text.StringBuilder(17);

        if (oldmac.Length < 12)
            return "00:00:00:00:00:00";

        newMac.Append(oldmac[10]);
        newMac.Append(oldmac[11]);
        newMac.Append(separator);
        newMac.Append(oldmac[8]);
        newMac.Append(oldmac[9]);
        newMac.Append(separator);
        newMac.Append(oldmac[6]);
        newMac.Append(oldmac[7]);
        newMac.Append(separator);
        newMac.Append(oldmac[4]);
        newMac.Append(oldmac[5]);
        newMac.Append(separator);
        newMac.Append(oldmac[2]);
        newMac.Append(oldmac[3]);
        newMac.Append(separator);
        newMac.Append(oldmac[0]);
        newMac.Append(oldmac[1]);

        return newMac.ToString();
    }

    public static IPAddress GetBroadcastAddress(IPAddress address, IPAddress subnetMask)
    {
        byte[] ipAdressBytes = address.GetAddressBytes();
        byte[] subnetMaskBytes = subnetMask.GetAddressBytes();

        if (ipAdressBytes.Length != subnetMaskBytes.Length)
        {
            MediaPortal.GUI.Library.Log.Error("Lengths of IP address and subnet mask do not match.");
            return IPAddress.Broadcast;
        }

        byte[] broadcastAddress = new byte[ipAdressBytes.Length];
        for (int i = 0; i < broadcastAddress.Length; i++)
        {
            broadcastAddress[i] = (byte)(ipAdressBytes[i] | (subnetMaskBytes[i] ^ 255));
        }
        return new IPAddress(broadcastAddress);
    }

    public static IPAddress GetHostIP(string hostName)
    {
        if (!System.Net.NetworkInformation.NetworkInterface.GetIsNetworkAvailable())
        {
            return null;
        }

        IPAddress lastAddress = null;
        try
        {
            IPAddress[] addressList = Dns.GetHostAddresses(hostName);
            foreach (IPAddress address in addressList)
            {
                MediaPortal.GUI.Library.Log.Debug(string.Format("Found host address '{0}' for {1}", address, hostName));
                lastAddress = address;
            }
        }
        catch (SocketException ex)
        {
            //some DNS error happened, return the message
            MediaPortal.GUI.Library.Log.Error(string.Format("DNS Error: {0}", ex.Message));
        }

        return lastAddress;
    }

    public static IPAddress GetSubnetMask(IPAddress address)
    {
        foreach (NetworkInterface adapter in NetworkInterface.GetAllNetworkInterfaces())
        {
            foreach (UnicastIPAddressInformation unicastIPAddressInformation in adapter.GetIPProperties().UnicastAddresses)
            {
                if (unicastIPAddressInformation.Address.AddressFamily == AddressFamily.InterNetwork)
                {
                    MediaPortal.GUI.Library.Log.Debug(string.Format("Found IP address '{0}', compared to '{1}'", unicastIPAddressInformation.Address, address));
                    if (address.Equals(unicastIPAddressInformation.Address))
                    {
                        MediaPortal.GUI.Library.Log.Debug(string.Format("Address '{0}' matched '{1}', returning mask {2}", unicastIPAddressInformation.Address, address, unicastIPAddressInformation.IPv4Mask));
                        return unicastIPAddressInformation.IPv4Mask;
                    }
                }
            }
        }
        MediaPortal.GUI.Library.Log.Error(string.Format("Can't find subnetmask for IP address '{0}'. Defaulting to 255.255.255.0", address));
        return IPAddress.Parse("255.255.255.0");
    }

#endregion


public class TestUtilities
{
    //Wandelt den angegebenen Ip-String in einen Int32-Wert um   
    [DllImport("Ws2_32.dll")]
    private static extern Int32 inet_addr(string ip);

    //Holt die Mac-Adresse der angegebenen Ip-Adresse   
    //Die Ip-Adresse muss im Int32-Format vorliegen.   
    [DllImport("Iphlpapi.dll")]
    private static extern int SendARP(Int32 dest, Int32 host, ref Int64 mac, ref Int32 len);

    //Diese Methode holt die Mac-Adresse einer angegebenen Ip-Adresse   
    //LocalIp ist die Ip-Adresse des eigenen PCs und kann ein Leerstring sein   
    //remoteIp ist die Ip-Adresse des anderen PCs und muss die durch Punkte getrennte
    //Ip-Adresse beinhalten   
    public static Int64 getRemoteMAC(string localIP, string remoteIP)
    {
      //Umwandlung der Ip-Adressen in Int32-Werte         
      Int32 ldest = inet_addr(remoteIP);
      Int32 lhost = inet_addr(localIP);

      try
      {
        Int64 macinfo = 0; //Dient zur Speicherung der Mac-Adresse             
        Int32 len = 6; //Länge             

        int res = SendARP(ldest, 0, ref macinfo, ref len); //Mac-Adresse holen

        return macinfo; //Int64 mit Mac-Adresse zurückgeben         
      }
      catch (Exception)
      {
        return 0;
      }
    }

    //Mit Hilfe dieser Methode kann die Mac-Adresse im Int64-Format leicht in
    //das übliche durch - getrennte Format umgewandelt werden     
    public static string formatMac(Int64 mac, char separator)
    {
      //Wenn Mac-Adresse 0 --> Umwandlung in 00-00-00-00-00-00 und im Anschluss exit
      if (mac <= 0)
        return "00-00-00-00-00-00";

      //Die Int64-Mac-Adresse wird in einen Hex-String umgewandelt, mit 0 aufgefüllt
      //und in ein Byte-Array aufgesplittet         
      char[] oldmac = Convert.ToString(mac, 16).PadLeft(12, '0').ToCharArray();

      System.Text.StringBuilder newMac = new System.Text.StringBuilder(17);

      if (oldmac.Length < 12)
        return "00-00-00-00-00-00";

      newMac.Append(oldmac[10]);
      newMac.Append(oldmac[11]);
      newMac.Append(separator);
      newMac.Append(oldmac[8]);
      newMac.Append(oldmac[9]);
      newMac.Append(separator);
      newMac.Append(oldmac[6]);
      newMac.Append(oldmac[7]);
      newMac.Append(separator);
      newMac.Append(oldmac[4]);
      newMac.Append(oldmac[5]);
      newMac.Append(separator);
      newMac.Append(oldmac[2]);
      newMac.Append(oldmac[3]);
      newMac.Append(separator);
      newMac.Append(oldmac[0]);
      newMac.Append(oldmac[1]);

      return newMac.ToString();
    }
  }
}
}
