﻿#region Copyright (C) 2005-2010 Team MediaPortal

// Copyright (C) 2005-2010 Team MediaPortal
// http://www.team-mediaportal.com
// 
// MediaPortal is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
// 
// MediaPortal is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with MediaPortal. If not, see <http://www.gnu.org/licenses/>.

#endregion


using System;
using System.Windows.Forms;
using System.Net;
using System.Xml;
using MediaPortal.GUI.Library;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;




namespace WOLPowerManager
{
    public partial class WOLPowerManagerConf : Form
    {
    #region Variables
        
        List<string> ids = new List<string>();
        
    #endregion

    #region Constructer
        
        public WOLPowerManagerConf()
        {
            InitializeComponent();
            //Lade Settings
            LoadSettings();
            
            //Lade Skinfiles in Combo
            new SkinInfo();
            getSkinFileList(ref ids);

            //Setze Versionsnummer
            lblVersion.Text = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString();
        }

    #endregion

    #region Save/Load

        private void LoadSettings()
        {
            try
            {
                //init
                comboRemoteButton.DataSource = GenerateActionList();
                comboRemoteButton.DisplayMember = "ActionName";
                comboRemoteButton.ValueMember = "ActionType";


                //load
                Settings.Load();
                
                ListBoxServer.Items.Clear();
                for (int i = 0; i < Settings.Servers.GetLength(0); i++)
                {
                    AddServer(i);
                }

                listViewPlugins.Items.Clear();
                for (int i = 0; i < Settings.Plugins.GetLength(0); i++)
                {
                    AddPlugin(i);
                }

                //Checkbox und RemoteButton Aktivieren/Deaktivieren
                chkManualWOL.Checked = Settings.ManualWOL;
                comboRemoteButton.Enabled = Settings.ManualWOL;

                comboRemoteButton.SelectedValue = Settings.RemoteButton;
                txtboxTimeout.Text = Settings.WaitTime.ToString();
                txtboxPort.Text = Settings.Port.ToString();
                chkUseIpInsteadDNS.Checked = Settings.UseIpInsteadDNSname;
                txtboxIntervalTime.Text = Settings.IntervalTime.ToString();
                ChkInterval.Checked = Settings.IntervalCheck;

            }

            catch (Exception ex)
            {
                Log.Error(ex);
            }
        }

        private void SaveSettings()
        {
            Settings.ManualWOL = chkManualWOL.Checked;

            Settings.WaitTime = Convert.ToInt32(txtboxTimeout.Text.ToString());
            Settings.Port = Convert.ToInt32(txtboxPort.Text.ToString());
            Settings.UseIpInsteadDNSname = chkUseIpInsteadDNS.Checked;
            Settings.RemoteButton = (MediaPortal.GUI.Library.Action.ActionType)comboRemoteButton.SelectedValue;
            Settings.IntervalCheck = ChkInterval.Checked;
            Settings.IntervalTime = Convert.ToInt32(txtboxIntervalTime.Text.ToString());
            Settings.Save();
        }

    #endregion
     
    #region GUI-HelpFuncs
        #region ServerTab

        private bool CheckIfEmpty()
        {
            if (txtboxComputername.Text == "" || txtboxIP.Text == "" || txtboxMAC.Text == "" || txtboxSubnet.Text == "")
                return true;
            else
                return false;
        }

        private void ClearEntries()
        {
            txtboxComputername.Text = "";
            txtboxIP.Text = "";
            txtboxMAC.Text = "";
            txtboxSubnet.Text = "";
        }

        private bool isValidMAC(string mac)
        {
            string pattern = "^([0-9a-fA-F][0-9a-fA-F]:){5}([0-9a-fA-F][0-9a-fA-F])$";

            return System.Text.RegularExpressions.Regex.IsMatch(mac, pattern);
        }

        private bool isValidServer()
        {
            if (CheckIfEmpty() == true) { MessageBox.Show("Error: Please enter an IP/Subnet/MAC-address!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation); return false; }
            if (isValidIP(txtboxIP.Text) == false) { MessageBox.Show("Error: Invalid IP-address format!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation); return false; }
            if (isValidIP(txtboxSubnet.Text) == false) { MessageBox.Show("Error: Invalid subnet-address format!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation); return false; }
            if (isValidMAC(txtboxMAC.Text) == false) { MessageBox.Show("Error: Invalid MAC-address format [xx:xx:xx:xx:xx:xx]!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation); return false; }
            return true;
        }

        /// <summary>
        /// method to validate an IP address
        /// using regular expressions. The pattern
        /// being used will validate an ip address
        /// with the range of 1.0.0.0 to 255.255.255.255
        /// </summary>
        /// <param name="addr">Address to validate</param>
        /// <returns></returns>
        public bool isValidIP(string addr)
        {
            //create our match pattern
            string pattern = @"^([1-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])(\.([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$";
            //return the results
            return System.Text.RegularExpressions.Regex.IsMatch(addr, pattern);
        }

        /// <summary>
        /// fügt einen Server zum Array Settings.Servers und ListboxServer hinzu
        /// </summary>
        /// <param name="nr">nullbasierender index der ersten Dimension, des hinzuzufügenden Servers, von Settings.Servers</param>
        private void AddServer(int nr)
        {
            ListBoxServer.Items.Insert(nr, Settings.Servers[nr, 0]);
            comboServer.Items.Insert(nr, Settings.Servers[nr, 0]);
        }

        private void DeleteServer(int nr)
        {
            //Eintrag aus Forms löschen.
            ListBoxServer.Items.RemoveAt(nr);
            comboServer.Items.RemoveAt(nr);
        }

        private void UpdateServer(int nr)
        {
            DeleteServer(nr);
            AddServer(nr);
        }

        #endregion


        #region PluginTab
        public string[] getSkinFileList(ref List<string> idList)
        {
            //lade Fileliste aus konfiguriertem Skinfolder
            string[] files = System.IO.Directory.GetFiles(SkinInfo.mpPaths.configuredSkinPath);
            foreach (string file in files)
            {
                try
                {
                    //Prüfe auf xml-Skinfile
                    if (file.StartsWith("common") == false && file.Contains("Dialog") == false && file.Contains("dialog") == false && file.Contains("wizard") == false && file.Contains("xml.backup") == false)
                    {
                        XmlDocument doc = new XmlDocument();
                        doc.Load(file);
                        XmlNode node = doc.DocumentElement.SelectSingleNode("/window/id");
                        idList.Add(node.InnerText);
                        comboPlugin.Items.Add(file.Remove(0, file.LastIndexOf(@"\") + 1).Replace(".xml", ""));
                    }
                }
                catch { }
            }
            return files;
        }

        private void AddPlugin(int nr)
        {
            //Item erstellen
            ListViewItem item = new ListViewItem();
            item.Checked = Convert.ToBoolean(Settings.Plugins[nr, 0]);
            item.Text = Settings.Plugins[nr, 1];
            item.SubItems.Add(Settings.Plugins[nr, 2]);
            //Item der Listview hinzufügen
            listViewPlugins.Items.Insert(nr,item);
        }

        private void UpdatePlugin(int nr)
        {
            DeletePlugin(nr);
            AddPlugin(nr);
        }

        private void DeletePlugin(int nr)
        {
            listViewPlugins.Items.RemoveAt(nr);
        }
        #endregion

        #region AdvancedTab

        private static List<ActionEntry> GenerateActionList()
        {
            var ret = new List<ActionEntry>();

            foreach (var variable in Enum.GetValues(typeof(MediaPortal.GUI.Library.Action.ActionType)))
                ret.Add(new ActionEntry((MediaPortal.GUI.Library.Action.ActionType)variable));

            ret.Sort();

            return ret;
        }

        #endregion
    #endregion


        #region GUI

        #region ServerTab

        private void ListBoxServer_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (ListBoxServer.SelectedIndices.Count != 0)
                {
                    btnServerEdit.Enabled = true;
                    btnServerDelete.Enabled = true;
                    txtboxComputername.Text = Settings.Servers[ListBoxServer.SelectedIndex, 0];
                    txtboxIP.Text = Settings.Servers[ListBoxServer.SelectedIndex, 1];
                    txtboxMAC.Text = Settings.Servers[ListBoxServer.SelectedIndex, 2];
                    chkStartupWOL.Checked = Convert.ToBoolean(Settings.Servers[ListBoxServer.SelectedIndex, 3]);
                    txtboxSubnet.Text = Settings.Servers[ListBoxServer.SelectedIndex, 4];
                }
                else
                {
                    btnServerEdit.Enabled = false;
                    btnServerDelete.Enabled = false;
                }
            }
            catch (Exception)
            {
                btnServerEdit.Enabled = false;
                btnServerDelete.Enabled = false;
            }
        }

        private int GetMacIp()
        {
            try
            {
                IPAddress hostIP = Utilities.GetHostIP(txtboxComputername.Text);
                if (txtboxIP.Text.Length < 1)
                    txtboxIP.Text = hostIP.ToString(); // only update if user hasn't manually entered an IP
                
                Int64 macinfo = Utilities.getRemoteMAC(txtboxIP.Text);
                string MacAdress = Utilities.formatMac(macinfo, ':');
                
                if (MacAdress == "00:00:00:00:00:00") 
                {
                    string prompt1 = string.Format("Error: Host seems to be powered off or is not reachable!\r\nTip: First power on the computer and check network connection to {0}, before getting MAC/IP", txtboxComputername.Text);
                    MessageBox.Show(prompt1, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return -1;
                }

                txtboxMAC.Text = MacAdress;

                //Get subnet mask of our local machine as a default setting. This will be fine for most users, but we'll allow users to enter their own mask
                //to permit waking across subnets.
                if (txtboxSubnet.Text.Length < 1)
                    txtboxSubnet.Text = Utilities.GetSubnetMask(Utilities.GetHostIP("")).ToString(); // only update if user hasn't manually entered a mask
                return 0;
            }
            catch (Exception)
            {
                MessageBox.Show("Error: Host can't be found!\r\nTip: Check your network connection or 'Computername'", "Error", MessageBoxButtons.OK,MessageBoxIcon.Exclamation);
                txtboxIP.Text="";
                txtboxMAC.Text = "";
                return -1;
            }
        }

        private void btnServerAdd_Click(object sender, EventArgs e)
        {
            //Prüfe ob Server bereits in Listbox
            for (int i = 0; i < Settings.Servers.GetLength(0); i++)
            {
                if (Settings.Servers[i, 0] == txtboxComputername.Text)
                {
                    MessageBox.Show("Server does already exist!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
            }

            //Ermittelt Mac/ip des neuen Computernamens und prüft
            if ((GetMacIp() == -1) || (isValidServer() == false)) return;

            //wenn Prüfungen erfolgreich, Array vergrößern und Daten eintragen
            Settings.Servers = Utilities.ResizeArray(Settings.Servers, Settings.Servers.GetLength(1), Settings.Servers.GetLength(0) + 1);
            Settings.Servers[Settings.Servers.GetLength(0)-1, 0] = txtboxComputername.Text;
            Settings.Servers[Settings.Servers.GetLength(0)-1, 1] = txtboxIP.Text;
            Settings.Servers[Settings.Servers.GetLength(0)-1, 2] = txtboxMAC.Text;
            Settings.Servers[Settings.Servers.GetLength(0)-1, 3] = chkStartupWOL.Checked.ToString();
            Settings.Servers[Settings.Servers.GetLength(0)-1, 4] = txtboxSubnet.Text;

            AddServer(Settings.Servers.GetLength(0)-1);

            //Lösche Einträge in Textfeldern
            ClearEntries();         
         }

        private void btnServerEdit_Click(object sender, EventArgs e)
        {

            //Ermittelt Mac/ip des neuen Computernamens und prüft
            if ((GetMacIp() == -1) || (isValidServer() == false)) return;

            //Aktualisiere in Plugins[,] alten, durch neuen Computername
            for (int i = 0; i < Settings.Plugins.GetLength(0); i++)
            {
                if (Settings.Plugins[i, 2] == Settings.Servers[ListBoxServer.SelectedIndex, 0])
                {
                    Settings.Plugins[i, 2] = txtboxComputername.Text;
                    UpdatePlugin(i);
                }
            }

            //Schreibe neue Daten in Array
            Settings.Servers[ListBoxServer.SelectedIndex, 0] = txtboxComputername.Text;
            Settings.Servers[ListBoxServer.SelectedIndex, 1] = txtboxIP.Text;
            Settings.Servers[ListBoxServer.SelectedIndex, 2] = txtboxMAC.Text;
            Settings.Servers[ListBoxServer.SelectedIndex, 3] = chkStartupWOL.Checked.ToString();
            Settings.Servers[ListBoxServer.SelectedIndex, 4] = txtboxSubnet.Text;

            //Aktualisiere Server-ListBox
            UpdateServer(ListBoxServer.SelectedIndex);

            //Lösche Einträge in Textfeldern
            ClearEntries();
        }

        private void btnServerDelete_Click(object sender, EventArgs e)
        {
            if (ListBoxServer.SelectedIndex != -1)
            {
                DialogResult result = MessageBox.Show("All plugins containing this server will be deleted from list! Are you sure?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                //Prüfe ob Server bereits als Server-Plugin Paar vorhanden ggf. Paare löschen
                if (result == DialogResult.Yes)
                {
                    int ArrayLength = Settings.Plugins.GetLength(0);
                    for (int i = ArrayLength-1; i >= 0; i--)
                    {
                        if (Settings.Servers[ListBoxServer.SelectedIndex, 0] == Settings.Plugins[i, 2])
                        {
                            Settings.Plugins = Utilities.RemoveAt(Settings.Plugins, i);
                            DeletePlugin(i);
                        }
                    }
                    Settings.Servers = Utilities.RemoveAt(Settings.Servers, ListBoxServer.SelectedIndex);
                    DeleteServer(ListBoxServer.SelectedIndex);

                    ClearEntries();
                }
            }
        }
        #endregion

        #region PluginTab

        private void listViewPlugins_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (listViewPlugins.SelectedIndices.Count != 0)
                {
                    btnEditPlugin.Enabled = true;
                    btnDeletePlugin.Enabled = true;
                    comboPlugin.SelectedItem = Settings.Plugins[listViewPlugins.SelectedIndices[0], 1];
                    comboServer.SelectedItem = Settings.Plugins[listViewPlugins.SelectedIndices[0], 2];
                }
                else
                {
                    btnEditPlugin.Enabled = false;
                    btnDeletePlugin.Enabled = false;
                }
            }
            catch (Exception)
            {
                btnEditPlugin.Enabled = false;
                btnDeletePlugin.Enabled = false;
            }

        }

        private void listViewPlugins_ItemChecked(object sender, ItemCheckedEventArgs e)
        {
            Settings.Plugins[e.Item.Index, 0] = Convert.ToString(e.Item.Checked);
        }

        private void btnAddPlugin_Click(object sender, EventArgs e)
        {
            //Prüfe ob Server-Plugin Paarung schon vorhanden
            for (int i = 0; i < Settings.Plugins.GetLength(0); i++)
            {
                if ((Settings.Plugins[i, 1] == comboPlugin.Text) && (Settings.Plugins[i, 2] == comboServer.Text))
                {
                    MessageBox.Show("Server-Plugin pair does already exist!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
            }
            
            if ((comboPlugin.SelectedIndex == -1) || (comboServer.SelectedIndex == -1))
            {
                MessageBox.Show("Please select Server-Plugin pair first!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            
            //Wenn Prüfung negativ, Array vergrößern und Auswahl in Array übertragen
            Settings.Plugins = Utilities.ResizeArray(Settings.Plugins, Settings.Plugins.GetLength(1), Settings.Plugins.GetLength(0) + 1);
            Settings.Plugins[Settings.Plugins.GetLength(0) - 1, 0] = Boolean.TrueString;
            Settings.Plugins[Settings.Plugins.GetLength(0) - 1, 1] = comboPlugin.GetItemText(comboPlugin.SelectedItem);
            Settings.Plugins[Settings.Plugins.GetLength(0) - 1, 2] = comboServer.GetItemText(comboServer.SelectedItem);
            Settings.Plugins[Settings.Plugins.GetLength(0) - 1, 3] = ids[comboPlugin.SelectedIndex];

            AddPlugin(Settings.Plugins.GetLength(0)-1);

            ClearEntries();
        }

        private void btnDeletePlugin_Click(object sender, EventArgs e)
        {
            Settings.Plugins = Utilities.RemoveAt(Settings.Plugins, listViewPlugins.SelectedIndices[0]);
            DeletePlugin(listViewPlugins.SelectedIndices[0]);
        }

        private void btnEditPlugin_Click(object sender, EventArgs e)
        {
            if (listViewPlugins.SelectedIndices.Count != 0)
            {
                Settings.Plugins[listViewPlugins.SelectedIndices[0], 1] = comboPlugin.GetItemText(comboPlugin.SelectedItem);
                Settings.Plugins[listViewPlugins.SelectedIndices[0], 2] = comboServer.GetItemText(comboServer.SelectedItem);
            }

            UpdatePlugin(listViewPlugins.SelectedIndices[0]);
        }

        #endregion

        #region AdvancedTab

        private void chkManualWOL_CheckedChanged(object sender, EventArgs e)
        {
            if (chkManualWOL.Checked==false)
                comboRemoteButton.Enabled = false;
            else
                comboRemoteButton.Enabled = true;
        }

        #endregion

        private void SaveButton_Click(object sender, EventArgs e)
        {
            SaveSettings();
            Close();
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void comboPlugin_SelectedIndexChanged(object sender, EventArgs e)
        {
            lblWinId.Text = ids[comboPlugin.SelectedIndex];
        }

        private void IntervalCheck_CheckedChanged(object sender, EventArgs e)
        {
            if (ChkInterval.Checked == false)
                txtboxIntervalTime.Enabled = false;
            else
                txtboxIntervalTime.Enabled = true;
        }

    #endregion

        #region internal-classes

        internal class ActionEntry : IComparable<ActionEntry>
        {
            public MediaPortal.GUI.Library.Action.ActionType ActionType { get; private set; }

            public string ActionName
            {
                get { return ActionType.ToString(); }
            }

            public int ActionID
            {
                get { return (int)ActionType; }
            }

            public ActionEntry(MediaPortal.GUI.Library.Action.ActionType actionType)
            {
                ActionType = actionType;
            }

            #region IComparable<ActionEntry> Member

            public int CompareTo(ActionEntry other)
            {
                return ActionName.CompareTo(other.ActionName);
            }

            #endregion
        }

        #endregion

        private void chkEditIP_CheckedChanged(object sender, EventArgs e)
        {
            if (chkEditIP.Checked)
                txtboxIP.Enabled = true;
            else
                txtboxIP.Enabled = false;
        }

        private void chkEditSubnet_CheckedChanged(object sender, EventArgs e)
        {
            if (chkEditSubnet.Checked)
                txtboxSubnet.Enabled = true;
            else
                txtboxSubnet.Enabled = false;
        }
    }
}
